@@ -1,26 +1,24 @@
 
 # fit the parameters of the bayesian network for a given network stucture.
 bn.fit.backend = function(x, data, cluster = NULL, method = "mle", extra.args,
-    keep.fitted = TRUE, debug = FALSE) {
-
-  # check which type of data we are dealing with.
-  type = data.type(data)
+    data.info, keep.fitted = TRUE, debug = FALSE) {
 
   # define the fitting functions.
-  if (type %in% discrete.data.types)
+  if (data.info$type %in% discrete.data.types)
     fit = bn.fit.backend.discrete
-  else if (type == "continuous")
+  else if (data.info$type == "continuous")
     fit = bn.fit.backend.continuous
-  else if (type == "mixed-cg")
+  else if (data.info$type == "mixed-cg")
     fit = bn.fit.backend.mixedcg
 
   # fit the parameters of each node.
   fitted = smartSapply(cluster, names(x$nodes), fit, dag = x, data = data,
-             method = method, extra.args = extra.args, keep.fitted = keep.fitted,
-             debug = debug)
+             method = method, extra.args = extra.args, data.info = data.info,
+             keep.fitted = keep.fitted, debug = debug)
   # preserve any additional class of the original bn object.
   orig.class = class(x)
-  class = c(orig.class[orig.class != "bn"], "bn.fit", fitted.from.data[[type]])
+  class = c(orig.class[orig.class != "bn"], "bn.fit",
+            fitted.from.data[[data.info$type]])
   # preserve the training node label from Bayesian network classifiers.
   if (x$learning$algo %in% classifiers)
     fitted = structure(fitted, class = class, training = x$learning$args$training)
@@ -32,7 +30,7 @@ bn.fit.backend = function(x, data, cluster = NULL, method = "mle", extra.args,
 }#BN.FIT.BACKEND
 
 bn.fit.backend.discrete = function(dag, node, data, method, extra.args,
-    keep.fitted = TRUE, debug = FALSE) {
+    data.info, keep.fitted = TRUE, debug = FALSE) {
 
   # store the labels of the parents and the children to get them only once.
   parents = dag$nodes[[node]]$parents
@@ -54,20 +52,30 @@ bn.fit.backend.discrete = function(dag, node, data, method, extra.args,
 
     # the parameters of the multinomial distribution are the probabilities
     # of the levels of the node and the configurations of its parents.
-    tab = minimal.table(data[, c(node, parents), drop = FALSE])
+    tab = minimal.table(data[, c(node, parents), drop = FALSE],
+            with.missing = !all(data.info$complete.nodes[c(node, parents)]))
+    # switch from joint to conditional probabilities.
+    tab = prop.table(tab, margin = seq(length(parents) + 1)[-1])
+
+    # replace unidentifiable conditional probabilities corresponding to
+    # unobserved parent configurations with uniform probabilites to prevent
+    # NAs from propagating.
+    if (extra.args$replace.unidentifiable)
+      tab[is.na(tab)] = 1/(nrow(tab))
 
   }#THEN
   else {
 
     # the parameters of the multinomial distribution are the expected values
     # of the corresponding parameters of the posterior Dirichlet distribution.
-    tab = minimal.table(data[, c(node, parents), drop = FALSE])
+    tab = minimal.table(data[, c(node, parents), drop = FALSE],
+            with.missing = !all(data.info$complete.nodes[c(node, parents)]))
     tab = tab + extra.args$iss / prod(dim(tab))
+    # switch from joint to conditional probabilities.
+    tab = prop.table(tab, margin = seq(length(parents) + 1)[-1])
 
   }#ELSE
 
-  # switch from joint to conditional probabilities.
-  tab = prop.table(tab, margin = seq(length(parents) + 1)[-1])
   # this is to preserve the ordering of the factor.
   class = ifelse(ordered.factor, "bn.fit.onode", "bn.fit.dnode")
 
@@ -85,7 +93,7 @@ bn.fit.backend.discrete = function(dag, node, data, method, extra.args,
 }#BN.FIT.BACKEND.DISCRETE
 
 bn.fit.backend.continuous = function(dag, node, data, method, extra.args,
-    keep.fitted = TRUE, debug = FALSE) {
+    data.info, keep.fitted = TRUE, debug = FALSE) {
 
   # cache the sample size.
   n = nrow(data)
@@ -103,18 +111,28 @@ bn.fit.backend.continuous = function(dag, node, data, method, extra.args,
 
   }#THEN
 
-  # the relevant quantities of the normal distribution are those usually found
-  # in an lm object: coefficients, fitted values and residuals, plus the
-  # residuals standard deviation.
+  if (method == "mle") {
+
+    # the relevant quantities of the normal distribution are those usually found
+    # in an lm object: coefficients, fitted values and residuals, plus the
+    # residuals standard deviation.
+    lmreg = fast.lm(data, node, parents, keep.fitted = keep.fitted,
+              with.missing = !all(data.info$complete.nodes[c(node, parents)]))
+
+    # replace unidentifiable regression coefficients with zeroes to prevent NAs
+    # from propagating.
+    if (extra.args$replace.unidentifiable)
+      lmreg$coefficients[is.na(lmreg$coefficients)] = 0
+
+  }#THEN
 
   structure(c(list(node = node, parents = parents, children = children),
-    fast.lm(data, node, parents, keep.fitted = keep.fitted)),
-    class = "bn.fit.gnode")
+    lmreg), class = "bn.fit.gnode")
 
 }#BN.FIT.BACKEND.CONTINUOUS
 
 bn.fit.backend.mixedcg = function(dag, node, data, method, extra.args,
-    keep.fitted = TRUE, debug = FALSE) {
+    data.info, keep.fitted = TRUE, debug = FALSE) {
 
   # cache the node's data.
   node.data = data[, node]
@@ -122,7 +140,8 @@ bn.fit.backend.mixedcg = function(dag, node, data, method, extra.args,
   # discrete nodes are parameterized by CPTs.
   if (is(node.data, "factor"))
     return(bn.fit.backend.discrete(dag = dag, node = node, data = data,
-      method = method, extra.args = extra.args, debug = debug))
+      method = method, extra.args = extra.args, data.info = data.info,
+      debug = debug))
 
   # store the labels of the parents and the children to get them only once.
   parents = dag$nodes[[node]]$parents
@@ -134,7 +153,7 @@ bn.fit.backend.mixedcg = function(dag, node, data, method, extra.args,
     # regression against the sample mean.
     return(bn.fit.backend.continuous(dag = dag, node = node, data = data,
       method = method, extra.args = extra.args, keep.fitted = keep.fitted,
-      debug = debug))
+      data.info = data.info, debug = debug))
 
   }#THEN
   else {
@@ -148,7 +167,7 @@ bn.fit.backend.mixedcg = function(dag, node, data, method, extra.args,
 
       return(bn.fit.backend.continuous(dag = dag, node = node, data = data,
         method = method, extra.args = extra.args, keep.fitted = keep.fitted,
-        debug = debug))
+        data.info = data.info, debug = debug))
 
     }#THEN
     else {
@@ -169,8 +188,18 @@ bn.fit.backend.mixedcg = function(dag, node, data, method, extra.args,
 
       # perform a linear regression for each configuration of the discrete
       # parents.
-      fitted = fast.cglm(data, node, parents = continuous.parents,
-                 configs = configs, keep.fitted = keep.fitted)
+      lmreg = fast.cglm(data, node, parents = continuous.parents,
+                configs = configs, keep.fitted = keep.fitted,
+                with.missing = !all(data.info$complete.nodes[c(node, parents)]))
+
+      # replace unidentifiable regression coefficients and standard errors with
+      # zeroes to prevent NAs from propagating.
+      if (extra.args$replace.unidentifiable) {
+
+        lmreg$coefficients[is.na(lmreg$coefficients)] = 0
+        lmreg$sd[is.na(lmreg$sd)] = 0
+
+      }#THEN
 
       if (keep.fitted) {
 
@@ -178,7 +207,7 @@ bn.fit.backend.mixedcg = function(dag, node, data, method, extra.args,
           dparents = which(parents %in% discrete.parents),
           gparents = which(parents %in% continuous.parents),
           dlevels = lapply(parents.data[parents %in% discrete.parents], levels)),
-          fitted), class = "bn.fit.cgnode")
+          lmreg), class = "bn.fit.cgnode")
 
       }#THEN
       else {
@@ -187,7 +216,7 @@ bn.fit.backend.mixedcg = function(dag, node, data, method, extra.args,
           dparents = which(parents %in% discrete.parents),
           gparents = which(parents %in% continuous.parents),
           dlevels = lapply(parents.data[parents %in% discrete.parents], levels)),
-          fitted), class = "bn.fit.cgnode")
+          lmreg), class = "bn.fit.cgnode")
 
       }#ELSE
 
