@@ -1,6 +1,6 @@
 
 incremental.association.optimized = function(x, whitelist, blacklist, test,
-  alpha, B, strict, debug = FALSE) {
+  alpha, B, max.sx = ncol(x), strict, complete, debug = FALSE) {
 
   nodes = names(x)
   mb2 = mb = list()
@@ -12,7 +12,8 @@ incremental.association.optimized = function(x, whitelist, blacklist, test,
 
     mb[[node]] = ia.markov.blanket(node, data = x, nodes = nodes,
          alpha = alpha, B = B, whitelist = whitelist, blacklist = blacklist,
-         backtracking = backtracking, test = test, debug = debug)
+         backtracking = backtracking, test = test, max.sx = max.sx,
+         complete = complete, debug = debug)
 
   }#FOR
 
@@ -27,7 +28,8 @@ incremental.association.optimized = function(x, whitelist, blacklist, test,
     # save results in a copy of mb;
     mb2[[node]] = neighbour(node, mb = mb, data = x, alpha = alpha,
          B = B, whitelist = whitelist, blacklist = blacklist,
-         backtracking = backtracking, test = test, debug = debug)
+         backtracking = backtracking, test = test, max.sx = max.sx,
+         complete = complete, debug = debug)
 
   }#FOR
 
@@ -42,14 +44,15 @@ incremental.association.optimized = function(x, whitelist, blacklist, test,
 }#INCREMENTAL.ASSOCIATION.OPTIMIZED
 
 incremental.association = function(x, cluster = NULL, whitelist, blacklist,
-  test, alpha, B, strict, debug = FALSE) {
+  test, alpha, B, max.sx = ncol(x), strict, complete, debug = FALSE) {
 
   nodes = names(x)
 
   # 1. [Compute Markov Blankets]
   mb = smartSapply(cluster, as.list(nodes), ia.markov.blanket, data = x,
          nodes = nodes, alpha = alpha, B = B, whitelist = whitelist,
-         blacklist = blacklist, test = test, debug = debug)
+         blacklist = blacklist, test = test, max.sx = max.sx,
+         complete = complete, debug = debug)
   names(mb) = nodes
 
   # check markov blankets for consistency.
@@ -58,7 +61,7 @@ incremental.association = function(x, cluster = NULL, whitelist, blacklist,
   # 2. [Compute Graph Structure]
   mb = smartSapply(cluster, as.list(nodes), neighbour, mb = mb, data = x,
          alpha = alpha, B = B, whitelist = whitelist, blacklist = blacklist,
-         test = test, debug = debug)
+         test = test, max.sx = max.sx, complete = complete, debug = debug)
   names(mb) = nodes
 
   # check neighbourhood sets for consistency.
@@ -69,7 +72,8 @@ incremental.association = function(x, cluster = NULL, whitelist, blacklist,
 }#INCREMENTAL.ASSOCIATION
 
 ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist,
-  start = character(0), backtracking = NULL, test, debug = FALSE) {
+  start = character(0), backtracking = NULL, test, max.sx = ncol(x), complete,
+  debug = FALSE) {
 
   nodes = nodes[nodes != x]
   known.good = known.bad = c()
@@ -126,13 +130,23 @@ ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist,
   # phase I (stepwise forward selection)
   repeat {
 
-    # stop if there are no nodes left.
+    # stop if there are no nodes left, or if we cannot add any more nodes
+    # because the conditioning set has grown too large.
     if (length(nodes) == 0 || is.null(nodes))
       break
 
+    if (length(mb) > max.sx) {
+
+       if (debug)
+         cat("  @ limiting conditioning sets to", max.sx, "nodes.\n")
+
+      break
+
+    }#THEN
+
     # get an association measure for each of the available nodes.
     association = indep.test(nodes, x, sx = mb, test = test, data = data,
-                    B = B, alpha = alpha)
+                    B = B, alpha = alpha, complete = complete)
 
     if (debug) {
 
@@ -177,7 +191,7 @@ ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist,
   fixed = fixed[fixed != ""]
 
   pv = roundrobin.test(x = x, z = mb, fixed = fixed, data = data, test = test,
-         B = B, alpha = alpha, debug = debug)
+         B = B, alpha = alpha, complete = complete, debug = debug)
 
   return(intersect(mb, c(names(pv[pv < alpha]), fixed)))
 
