@@ -2,7 +2,7 @@
 # convert a set of arcs to a (real) adjacency matrix.
 arcs2amat = function(arcs, nodes) {
 
-  .Call("arcs2amat",
+  .Call(call_arcs2amat,
         arcs = as.character(arcs),
         nodes = as.character(nodes))
 
@@ -11,7 +11,7 @@ arcs2amat = function(arcs, nodes) {
 # convert an adjacency matrix back to a set of arcs.
 amat2arcs = function(a, nodes) {
 
-  .Call("amat2arcs",
+  .Call(call_amat2arcs,
         amat = a,
         nodes = nodes)
 
