@@ -9,7 +9,7 @@ int length_names = 0;
 SEXP arcs, temp, names;
 
   /* get the names of the nodes. */
-  names = getAttrib(nbr, R_NamesSymbol);
+  PROTECT(names = getAttrib(nbr, R_NamesSymbol));
   length_names = length(names);
 
   /* scan the structure to determine the number of arcs.  */
@@ -31,7 +31,7 @@ SEXP arcs, temp, names;
     /* set the column names. */
     setDimNames(arcs, R_NilValue, mkStringVec(2, "from", "to"));
 
-    UNPROTECT(1);
+    UNPROTECT(2);
 
     return arcs;
 
@@ -62,7 +62,7 @@ SEXP arcs, temp, names;
 
   }/*FOR*/
 
-  UNPROTECT(1);
+  UNPROTECT(2);
 
   return arcs;
 
@@ -76,13 +76,18 @@ int *t = NULL, *c = NULL;
 SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash;
 
   /* get the node set of each network. */
-  tnodes = getAttrib(getListElement(target, "nodes"), R_NamesSymbol);
-  cnodes = getAttrib(getListElement(current, "nodes"), R_NamesSymbol);
+  PROTECT(tnodes = getAttrib(getListElement(target, "nodes"), R_NamesSymbol));
+  PROTECT(cnodes = getAttrib(getListElement(current, "nodes"), R_NamesSymbol));
 
   /* first check: node sets must have the same size. */
-  if (length(tnodes) != length(cnodes))
+  if (length(tnodes) != length(cnodes)) {
+
+    UNPROTECT(2);
+
     return mkString("Different number of nodes");
 
+  }/*THEN*/
+
   /* store for future use. */
   nnodes = length(tnodes);
 
@@ -99,7 +104,7 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash;
 
     if (c[i] != i + 1) {
 
-      UNPROTECT(1);
+      UNPROTECT(3);
 
       return mkString("Different node sets");
 
@@ -114,9 +119,14 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash;
   carcs = getListElement(current, "arcs");
 
   /* third check: arc sets must have the same size. */
-  if (length(tarcs) != length(carcs))
+  if (length(tarcs) != length(carcs)) {
+
+    UNPROTECT(2);
+
     return mkString("Different number of directed/undirected arcs");
 
+  }/*THEN*/
+
   /* store for future use. */
   narcs = length(tarcs)/2;
 
@@ -137,7 +147,7 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash;
     /* compare the integer vectors as generic memory areas. */
     if (memcmp(t, c, narcs * sizeof(int))) {
 
-      UNPROTECT(2);
+      UNPROTECT(4);
 
       return mkString("Different arc sets");
 
@@ -147,6 +157,8 @@ SEXP tnodes, cnodes, cmatch, tarcs, carcs, thash, chash;
 
   }/*THEN*/
 
+  UNPROTECT(2);
+
   /* all checks completed successfully, returning TRUE. */
   return ScalarLogical(TRUE);
 
