@@ -24,13 +24,14 @@ extern SEXP cgsd(SEXP, SEXP, SEXP);
 extern SEXP chow_liu(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP class_err(SEXP, SEXP);
 extern SEXP configurations(SEXP, SEXP, SEXP);
+extern SEXP count_observed_values(SEXP);
 extern SEXP cpdag(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP cpdist_lw(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP dag2ug(SEXP, SEXP, SEXP);
 extern SEXP data_frame_finite(SEXP);
 extern SEXP data_type(SEXP);
 extern SEXP dataframe_column(SEXP, SEXP, SEXP);
-extern SEXP dedup(SEXP, SEXP, SEXP);
+extern SEXP dedup(SEXP, SEXP, SEXP, SEXP);
 extern SEXP dpred(SEXP, SEXP, SEXP, SEXP);
 extern SEXP elist2arcs(SEXP);
 extern SEXP empty_graph(SEXP, SEXP);
@@ -47,7 +48,7 @@ extern SEXP hc_opt_step(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SE
 extern SEXP hc_to_be_added(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP ide_cozman_graph(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP increment_test_counter(SEXP);
-extern SEXP indep_test(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP indep_test(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP is_dag(SEXP, SEXP);
 extern SEXP is_listed(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP is_pdag_acyclic(SEXP, SEXP, SEXP, SEXP, SEXP);
